#!/usr/bin/env python


print("Es wird versucht zu einer Waterkotte Wärempumpe mit WCPU Steuerung über Modbus TCP eine Verbinung herzustellen.")
ip_address = input("Bitte gib die IP-Adresse des Mobus TP Gateways ein: ")

try:
    from pymodbus.client import ModbusTcpClient
except ImportError:
    print("Fehler: pymodbus ist nicht installiert. Bitte installiere es mit 'pip install pymodbus'.")
    sys.exit(1)


import struct
from datetime import datetime
import time


# Definition der Datentypen
TYPE_BOOL = "bool"  
TYPE_INT8 = "int8"
TYPE_BYTE = "byte"
TYPE_UINT8 = "uint8"
TYPE_INT16 = "int16"
TYPE_UINT16 = "uint16"
TYPE_INT32 = "int32"
TYPE_UINT32 = "uint32"
TYPE_INT64 = "int64"
TYPE_UINT64 = "uint64"
TYPE_FLOAT = "float32"
TYPE_FLOAT32 = "float32"
TYPE_FLOAT64 = "float64"
TYPE_STRING = "string"
TYPE_TIME = "time"
TYPE_DATETIME = "datetimeType"
TYPE_REGISTERS = "registers"
TYPE_BITS = "bits"


class ModbusReader:
    def __init__(self, host: str, port: int = 502):
        self.client = ModbusTcpClient(host, port=port)
        
        if not self.client.connect():
            raise Exception(f"Verbindung zum Modbus-Server bei {host}:{port} fehlgeschlagen!")

    def read_value(self, function: int, addr: int, size: int, value_type: str):
        if function == 1:  # Funktionscode 1 - Read Coils
            result = self.client.read_coils(address=addr, count=size)
        elif function == 3:  # Funktionscode 3 - Read Holding Registers
            result = self.client.read_holding_registers(address=addr, count=size)
        elif function == 4:  # Funktionscode 4 - Read Input Registers
            result = self.client.read_input_registers(address=addr, count=size)
        else:
            raise ValueError(f"Unsupported function code: {function}")
        
        if not result.isError():
            registers = result.bits if function == 1 else result.registers
            value = self.convert_value(registers, value_type)
            return value
        else:
            raise Exception(f"Fehler beim Lesen der Register {addr}")

    def convert_value(self, registers, value_type: str):
        if value_type == TYPE_BOOL:
            # Umwandlung von Coils (Einzel-Bit-Status) zu bool
            return registers[0]
        elif value_type == TYPE_INT8:
            return struct.unpack('b', struct.pack('H', registers[0]))[0]
        elif value_type == TYPE_UINT8:
            return registers[0]
        elif value_type == TYPE_BYTE:
            return registers[0]
        elif value_type == TYPE_REGISTERS:
            return registers
        elif value_type == TYPE_INT16:
            if len(registers) >= 2:
                return struct.unpack('h', struct.pack('HH', registers[0], registers[1]))[0]
            else:
                raise ValueError("Nicht genug Register für einen 16-Bit-Wert")
        elif value_type == TYPE_TIME:
            return_value = "{:02d}:{:02d}".format(registers[0] >> 8, registers[0] & 0xFF)
            return return_value
        elif value_type == TYPE_UINT16:
            if len(registers) >= 2:
                return struct.unpack('H', struct.pack('HH', registers[0], registers[1]))[0]
            else:
                raise ValueError("Nicht genug Register für einen 16-Bit-Wert")
        elif value_type == TYPE_INT32:
            if len(registers) >= 4:
                return struct.unpack('i', struct.pack('HHHH', registers[0], registers[1], registers[2], registers[3]))[0]
            else:
                raise ValueError("Nicht genug Register für einen 32-Bit-Wert")
        elif value_type == TYPE_UINT32:
            if len(registers) >= 4:
                return struct.unpack('I', struct.pack('HHHH', registers[0], registers[1], registers[2], registers[3]))[0]
            else:
                raise ValueError("Nicht genug Register für einen 32-Bit-Wert")
        elif value_type == TYPE_INT64:
            if len(registers) >= 8:
                return struct.unpack('q', struct.pack('HHHHHHHH', *registers))[0]
            else:
                raise ValueError("Nicht genug Register für einen 64-Bit-Wert")
        elif value_type == TYPE_UINT64:
            if len(registers) >= 8:
                return struct.unpack('Q', struct.pack('HHHHHHHH', *registers))[0]
            else:
                raise ValueError("Nicht genug Register für einen 64-Bit-Wert")
        elif value_type == TYPE_FLOAT32:
            if len(registers) >= 2:
                return struct.unpack('f', struct.pack('HH', registers[0], registers[1]))[0]
            else:
                raise ValueError("Nicht genug Register für einen 32-Bit Float-Wert")
        elif value_type == TYPE_FLOAT64:
            if len(registers) >= 4:
                return struct.unpack('d', struct.pack('HHHH', registers[0], registers[1], registers[2], registers[3]))[0]
            else:
                raise ValueError("Nicht genug Register für einen 64-Bit Float-Wert")

        elif value_type == TYPE_DATETIME:
            if len(registers) < 6:
                raise ValueError("Nicht genug Register für Zeit und Datum.")
    
            # Zeit extrahieren
            second = registers[0]  # Sekunden
            minute = registers[1]  # Minuten
            hour = registers[2]    # Stunden
    
            # Datum extrahieren
            day = registers[3]         # Tag
            month = registers[4] + 1   # Monat (um 1 subtrahieren)
            year = registers[5] + 1900 # Jahr (um 1900 addieren)
    
   
            dt = datetime(year, month, day, hour, minute, second)
            return dt

        elif value_type == TYPE_BITS:
            if len(registers) >= 1:  # Mindestens ein Register erforderlich
                # Nur die ersten 8 Bits des ersten Registers extrahieren
                bitfield = registers[0] & 0xFF  # Maske für die unteren 8 Bits
                bits = [(bitfield >> i) & 1 == 1 for i in range(8)]  # Extrahiere jedes Bit als bool
                return bits
            else:
                raise ValueError("Nicht genug Register für TYPE_BITS")
        elif value_type == TYPE_STRING:
            return_value = ""

            # Iteriere über alle Register
            for value in registers:
                # Niedriges Byte extrahieren und als Zeichen hinzufügen
                return_value += chr(value & 0xFF)
                # Hohes Byte extrahieren und als Zeichen hinzufügen
                return_value += chr(value >> 8)

            return return_value
        else:
            raise ValueError(f'Unbekannter Datentyp: {value_type}')


def read_and_print_value(modbus_reader, function, addr, size, value_type, caption, unit=''):
    value = modbus_reader.read_value(function=function, addr=addr, size=size, value_type=value_type)
    time.sleep(1)
    print(f"{hex(addr)} - {caption}: {value} {unit}")






modbus_reader = ModbusReader(ip_address)
    
input("Keine Fehlermeldung bisher? Die Verbindung zum Gateway sollte stehen. Drücke Enter um weiter zu machen,....")

print("Ich versuche jetzt einen ersten Wert zu lesen:")
read_and_print_value(modbus_reader, function=1, addr=0x0661, size=1, value_type=TYPE_BOOL, caption='Water', unit="")
read_and_print_value(modbus_reader, function=1, addr=0x03F1, size=1, value_type=TYPE_BOOL, caption='Heat', unit="")


print("Oben die Werte ob Warmwasser und Heizung aktiviert sind (FALSE) oder ob die händishce Abschaltung ein ist (TRUE).")
input("Wenn das läuft, dann gehen wir aufs Ganze...(Enter)")





# ### Jetzt alles
# 
# Hier werden die meisten Register ausgelesen. Nachdem ich keinen Bedarf hatte, ist Kühlung, Solar und Pool ausgelassen

# In[17]:


    
read_and_print_value(modbus_reader, function=3, addr=0x01B1, size=2, value_type=TYPE_FLOAT32, caption='Outdoor Temp.', unit="°C")
read_and_print_value(modbus_reader, function=3, addr=0x01D1, size=2, value_type=TYPE_FLOAT32, caption='Outdoor Temp. (1h)', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x01F1, size=2, value_type=TYPE_FLOAT32, caption='Outdoor Temp. (24h)', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0211, size=2, value_type=TYPE_FLOAT32, caption='Source in Temp', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0231, size=2, value_type=TYPE_FLOAT32, caption='Source out Temp', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0251, size=2, value_type=TYPE_FLOAT32, caption='T evaporator', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0271, size=2, value_type=TYPE_FLOAT32, caption='T suction line', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0291, size=2, value_type=TYPE_FLOAT32, caption='p evaporator')
read_and_print_value(modbus_reader, function=4, addr=0x02B1, size=2, value_type=TYPE_FLOAT32, caption='Rücklauf target', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x02D1, size=2, value_type=TYPE_FLOAT32, caption='Rücklauf real', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x02F1, size=2, value_type=TYPE_FLOAT32, caption='T heating out', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0311, size=2, value_type=TYPE_FLOAT32, caption='T condenser', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0331, size=2, value_type=TYPE_FLOAT32, caption='p condenser')
read_and_print_value(modbus_reader, function=3, addr=0x0351, size=2, value_type=TYPE_FLOAT32, caption='Pilotroom ', unit='°C')
read_and_print_value(modbus_reader, function=4, addr=0x0371, size=2, value_type=TYPE_FLOAT32, caption='Pilotroom 1h', unit='°C')
read_and_print_value(modbus_reader, function=4, addr=0x0391, size=2, value_type=TYPE_FLOAT32, caption='Warmwasser', unit='°C')
read_and_print_value(modbus_reader, function=4, addr=0x03B1, size=2, value_type=TYPE_FLOAT32, caption='Pool', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x03D1, size=2, value_type=TYPE_FLOAT32, caption='Solar', unit='°C')

# Heizungswerte
read_and_print_value(modbus_reader, function=1, addr=0x03F1, size=1, value_type=TYPE_BOOL, caption='Heating turn off')
read_and_print_value(modbus_reader, function=4, addr=0x0401, size=1, value_type=TYPE_TIME, caption='Heating time start')
read_and_print_value(modbus_reader, function=4, addr=0x0411, size=1, value_type=TYPE_TIME, caption='Heating time stop')
read_and_print_value(modbus_reader, function=4, addr=0x0421, size=2, value_type=TYPE_FLOAT32, caption='T out begin')
read_and_print_value(modbus_reader, function=4, addr=0x0441, size=0x2, value_type=TYPE_FLOAT32, caption='T return base')
read_and_print_value(modbus_reader, function=4, addr=0x0461, size=0x2, value_type=TYPE_FLOAT32, caption='T curve gradient')
read_and_print_value(modbus_reader, function=4, addr=0x0481, size=0x2, value_type=TYPE_FLOAT32, caption='T curve limit')
read_and_print_value(modbus_reader, function=4, addr=0x04A1, size=0x2, value_type=TYPE_FLOAT32, caption='T return real')
read_and_print_value(modbus_reader, function=4, addr=0x04C1, size=0x2, value_type=TYPE_FLOAT32, caption='T return target')
read_and_print_value(modbus_reader, function=4, addr=0x04E1, size=0x2, value_type=TYPE_FLOAT32, caption='Heating Hysteresis')
read_and_print_value(modbus_reader, function=4, addr=0x0501, size=0x2, value_type=TYPE_FLOAT32, caption='T pilot room target')
read_and_print_value(modbus_reader, function=4, addr=0x0521, size=0x1, value_type=TYPE_UINT8, caption='T pilot room influence')
read_and_print_value(modbus_reader, function=1, addr=0x0531, size=0x1, value_type=TYPE_BOOL, caption='Increase Off')
read_and_print_value(modbus_reader, function=4, addr=0x0541, size=0x1, value_type=TYPE_TIME, caption='Set pt chg start')
read_and_print_value(modbus_reader, function=4, addr=0x0551, size=0x1, value_type=TYPE_TIME, caption='Set pt chg stop')
read_and_print_value(modbus_reader, function=4, addr=0x0561, size=0x2, value_type=TYPE_FLOAT32, caption='Setpoint change')
read_and_print_value(modbus_reader, function=4, addr=0x0581, size=0x1, value_type=TYPE_UINT8, caption='Step mode')
read_and_print_value(modbus_reader, function=4, addr=0x0591, size=0x2, value_type=TYPE_FLOAT32, caption='Step max. diff.')

# Warmwasser

read_and_print_value(modbus_reader, function=1, addr=0x0661, size=1, value_type=TYPE_BOOL, caption='Hot Water turn off')
read_and_print_value(modbus_reader, function=4, addr=0x0671, size=1, value_type=TYPE_TIME, caption='Time start')
read_and_print_value(modbus_reader, function=4, addr=0x0681, size=1, value_type=TYPE_TIME, caption='Time stop')
read_and_print_value(modbus_reader, function=4, addr=0x0691, size=2, value_type=TYPE_FLOAT32, caption='T water real')
read_and_print_value(modbus_reader, function=4, addr=0x06B1, size=2, value_type=TYPE_FLOAT32, caption='T water target')
read_and_print_value(modbus_reader, function=4, addr=0x06D1, size=2, value_type=TYPE_FLOAT32, caption='Domestic Water Hysteresis')
read_and_print_value(modbus_reader, function=3, addr=0x06F1, size=1, value_type=TYPE_BYTE, caption='Legio. MTWTFSSP')
read_and_print_value(modbus_reader, function=4, addr=0x0701, size=1, value_type=TYPE_TIME, caption='Legio. Begin')
read_and_print_value(modbus_reader, function=4, addr=0x0711, size=1, value_type=TYPE_TIME, caption='LEGIO. End')
read_and_print_value(modbus_reader, function=4, addr=0x0721, size=2, value_type=TYPE_FLOAT, caption='Legio. Nominal')
read_and_print_value(modbus_reader, function=4, addr=0x0741, size=1, value_type=TYPE_BYTE, caption='Num. of compressors')
read_and_print_value(modbus_reader, function=4, addr=0x0751, size=1, value_type=TYPE_BYTE, caption='Time delay on solar')

# MixerValve
read_and_print_value(modbus_reader, function=4, addr=0x08D1, size=2, value_type=TYPE_FLOAT, caption='T circuit')
read_and_print_value(modbus_reader, function=4, addr=0x08F1, size=1, value_type=TYPE_TIME, caption='Time start')
read_and_print_value(modbus_reader, function=4, addr=0x0901, size=1, value_type=TYPE_TIME, caption='Time stop')
read_and_print_value(modbus_reader, function=3, addr=0x0911, size=2, value_type=TYPE_FLOAT, caption='T out base')
read_and_print_value(modbus_reader, function=4, addr=0x0931, size=2, value_type=TYPE_FLOAT, caption='T curve gradient')
read_and_print_value(modbus_reader, function=4, addr=0x0951, size=2, value_type=TYPE_FLOAT, caption='T curve limit')
read_and_print_value(modbus_reader, function=4, addr=0x0971, size=2, value_type=TYPE_FLOAT, caption='T nominal')
read_and_print_value(modbus_reader, function=4, addr=0x0991, size=2, value_type=TYPE_FLOAT, caption='Kp')
read_and_print_value(modbus_reader, function=4, addr=0x09B1, size=1, value_type=TYPE_BYTE, caption='Tn')
read_and_print_value(modbus_reader, function=4, addr=0x09C1, size=1, value_type=TYPE_BYTE, caption='Ta')
read_and_print_value(modbus_reader, function=4, addr=0x09D1, size=2, value_type=TYPE_FLOAT, caption='Mixer Hysteresis')
read_and_print_value(modbus_reader, function=4, addr=0x09F1, size=2, value_type=TYPE_BYTE, caption='Mixer mode (AUTO:OFF:DOWN:UP)')
read_and_print_value(modbus_reader, function=4, addr=0x0A01, size=1, value_type=TYPE_BYTE, caption='DO state')

# AuxiliaryHeat
read_and_print_value(modbus_reader, function=4, addr=0x0A11, size=1, value_type=TYPE_BYTE, caption='Step 2 mode (OFF:LEGIONELLA:ON)')
read_and_print_value(modbus_reader, function=4, addr=0x0A21, size=2, value_type=TYPE_FLOAT, caption='LimitHeatSrcIn')

# OperatingHours
read_and_print_value(modbus_reader, function=4, addr=0x0A41, size=2, value_type=TYPE_FLOAT, caption='Compressor 1', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0A61, size=2, value_type=TYPE_FLOAT, caption='Compressor 2', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0A81, size=2, value_type=TYPE_FLOAT, caption='Heating comp.', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0AA1, size=2, value_type=TYPE_FLOAT, caption='Heating aux.', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0AC1, size=2, value_type=TYPE_FLOAT, caption='Cooling', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0AE1, size=2, value_type=TYPE_FLOAT, caption='Hotwater comp.', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0B01, size=2, value_type=TYPE_FLOAT, caption='Hotwater aux.', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0B21, size=2, value_type=TYPE_FLOAT, caption='Pool', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0B41, size=2, value_type=TYPE_FLOAT, caption='Solar thermal collector', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0B61, size=1, value_type=TYPE_BYTE, caption='Operating hours (None:Compressor1:Compressor2:HeatingCompr:DWaterCompr:HeatingAux:DWaterAux:PoolCompr:Cooling:Solar thermal collector)')
read_and_print_value(modbus_reader, function=4, addr=0x0B71, size=2, value_type=TYPE_FLOAT, caption='New value', unit='h')



# Failure TimeStamp
read_and_print_value(modbus_reader, function=4, addr=0x0B91, size=0xD, value_type=TYPE_DATETIME, caption='Time')
read_and_print_value(modbus_reader, function=4, addr=0x0C61, size=0xD, value_type=TYPE_DATETIME, caption='Date')

# Failure Operating Mode
read_and_print_value(modbus_reader, function=1, addr=0x0D31, size=0x8, value_type=TYPE_BITS, caption='Running mode')

# Failure DO/DI Buffer
read_and_print_value(modbus_reader, function=1, addr=0x0D41, size=0x8, value_type=TYPE_BITS, caption='DO buffer')
read_and_print_value(modbus_reader, function=1, addr=0x0D51, size=0x8, value_type=TYPE_BITS, caption='DI buffer')

# Failure AI Error
read_and_print_value(modbus_reader, function=1, addr=0x0D61, size=0x10, value_type=TYPE_BITS, caption='Sensor error')

# Failure Temperatures
read_and_print_value(modbus_reader, function=4, addr=0x0D71, size=0x2, value_type=TYPE_FLOAT, caption='P evaporator')
read_and_print_value(modbus_reader, function=4, addr=0x0D91, size=0x2, value_type=TYPE_FLOAT, caption='p condenser')
read_and_print_value(modbus_reader, function=4, addr=0x0DB1, size=0x2, value_type=TYPE_FLOAT, caption='T return real')
read_and_print_value(modbus_reader, function=4, addr=0x0DD1, size=0x2, value_type=TYPE_FLOAT, caption='T heating out')
read_and_print_value(modbus_reader, function=4, addr=0x0DF1, size=0x2, value_type=TYPE_FLOAT, caption='T source in')
read_and_print_value(modbus_reader, function=4, addr=0x0E11, size=0x2, value_type=TYPE_FLOAT, caption='T source out')
read_and_print_value(modbus_reader, function=4, addr=0x0E31, size=0x2, value_type=TYPE_FLOAT, caption='T suction line')
read_and_print_value(modbus_reader, function=4, addr=0x0E51, size=0x2, value_type=TYPE_FLOAT, caption='T outdoor')
read_and_print_value(modbus_reader, function=4, addr=0x0E71, size=0x2, value_type=TYPE_FLOAT, caption='T hot water')
read_and_print_value(modbus_reader, function=4, addr=0x0E91, size=0x2, value_type=TYPE_FLOAT, caption='T pool')
read_and_print_value(modbus_reader, function=4, addr=0x0EB1, size=0x2, value_type=TYPE_FLOAT, caption='T solar')

# Service Information
read_and_print_value(modbus_reader, function=4, addr=0x0ED1, size=0x1, value_type=TYPE_BYTE, caption='View (Normal:Expert)')
read_and_print_value(modbus_reader, function=4, addr=0x0EE1, size=0x2, value_type=TYPE_REGISTERS, caption='PIN-code')
read_and_print_value(modbus_reader, function=4, addr=0x0F01, size=0x1, value_type=TYPE_BYTE, caption='Factory reset')
read_and_print_value(modbus_reader, function=4, addr=0x0F11, size=0x1, value_type=TYPE_BOOL, caption='Software Reset')
read_and_print_value(modbus_reader, function=1, addr=0x0F21, size=0x10, value_type=TYPE_BITS, caption='DO-manual on')
read_and_print_value(modbus_reader, function=4, addr=0x0F31, size=0x1, value_type=TYPE_BOOL, caption='First heating 7d')
read_and_print_value(modbus_reader, function=4, addr=0x0F41, size=0x2, value_type=TYPE_FLOAT, caption='First heating temp')
read_and_print_value(modbus_reader, function=4, addr=0x0F61, size=0x1, value_type=TYPE_BYTE, caption='SourcePumpPreRunPeriod')
read_and_print_value(modbus_reader, function=4, addr=0x0F71, size=0x1, value_type=TYPE_BYTE, caption='HeatingPumpPreRunPeriod')
read_and_print_value(modbus_reader, function=4, addr=0x0F81, size=0x1, value_type=TYPE_BYTE, caption='After run period')
read_and_print_value(modbus_reader, function=4, addr=0x0F91, size=0x1, value_type=TYPE_BYTE, caption='Cyclic Measurement')
read_and_print_value(modbus_reader, function=4, addr=0x0FA1, size=0x2, value_type=TYPE_FLOAT, caption='Offset press low')
read_and_print_value(modbus_reader, function=4, addr=0x0FC1, size=0x2, value_type=TYPE_FLOAT, caption='Offset press high')
read_and_print_value(modbus_reader, function=4, addr=0x0FE1, size=0x1, value_type=TYPE_BYTE, caption='ND Status')

# Error Events
read_and_print_value(modbus_reader, function=1, addr=0x0FF1, size=0x10, value_type=TYPE_BITS, caption='Failure')
read_and_print_value(modbus_reader, function=1, addr=0x1001, size=0x10, value_type=TYPE_BITS, caption='Interruption')
read_and_print_value(modbus_reader, function=1, addr=0x1011, size=0x10, value_type=TYPE_BITS, caption='Warning source')
read_and_print_value(modbus_reader, function=1, addr=0x1021, size=0x10, value_type=TYPE_BITS, caption='Warning outlet')
read_and_print_value(modbus_reader, function=1, addr=0x1031, size=0x10, value_type=TYPE_BITS, caption='Warning sensors')
read_and_print_value(modbus_reader, function=1, addr=0x1041, size=0x10, value_type=TYPE_BITS, caption='Warning diff.')
read_and_print_value(modbus_reader, function=1, addr=0x1051, size=0x10, value_type=TYPE_BITS, caption='Suppr.warn.sourc')
read_and_print_value(modbus_reader, function=1, addr=0x1061, size=0x10, value_type=TYPE_BITS, caption='Suppr.warn.outl.')
read_and_print_value(modbus_reader, function=1, addr=0x1071, size=0x10, value_type=TYPE_BITS, caption='Suppr.warn.sens.')
read_and_print_value(modbus_reader, function=1, addr=0x1081, size=0x10, value_type=TYPE_BITS, caption='Suppr. warn diff.')

# Default Services
read_and_print_value(modbus_reader, function=1, addr=0x1091, size=0x1, value_type=TYPE_BOOL, caption='Heating service')
read_and_print_value(modbus_reader, function=1, addr=0x10A1, size=0x1, value_type=TYPE_BOOL, caption='Cooling service')
read_and_print_value(modbus_reader, function=1, addr=0x10B1, size=0x1, value_type=TYPE_BOOL, caption='Hot water service')
read_and_print_value(modbus_reader, function=1, addr=0x10C1, size=0x1, value_type=TYPE_BOOL, caption='Pool service')
read_and_print_value(modbus_reader, function=1, addr=0x10D1, size=0x1, value_type=TYPE_BOOL, caption='Solar service')
read_and_print_value(modbus_reader, function=1, addr=0x10E1, size=0x1, value_type=TYPE_BOOL, caption='Step 2 service')
read_and_print_value(modbus_reader, function=1, addr=0x10F1, size=0x1, value_type=TYPE_BOOL, caption='Mixer mode service')
read_and_print_value(modbus_reader, function=4, addr=0x1101, size=0x1, value_type=TYPE_BYTE, caption='Heat source Tmin (+2 °C: -1 °C:-9 °C)')
read_and_print_value(modbus_reader, function=4, addr=0x1111, size=1, value_type=TYPE_BYTE, caption='Solar sensor (Pt500:Pt1000)', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1121, size=1, value_type=TYPE_BYTE, caption='Refrigerant (R407c:R134a)', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1131, size=1, value_type=TYPE_BYTE, caption='Num. Compr. (1:1_75_100:1_66_100:1_50_100:2)', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1141, size=1, value_type=TYPE_BYTE, caption='Modem Type (NONE:WPCU.M56k)', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1151, size=1, value_type=TYPE_BYTE, caption='Comm. mode (NO-COMM:LINE-MODBUS:DIAL-MODBUS)', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1161, size=1, value_type=TYPE_BYTE, caption='Ringtones', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1171, size=2, value_type=TYPE_REGISTERS, caption='External control', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1191, size=2, value_type=TYPE_FLOAT, caption='P<sub>electric</sub>', unit='kW')
read_and_print_value(modbus_reader, function=4, addr=0x11B1, size=2, value_type=TYPE_FLOAT, caption='P<sub>thermal</sub>', unit='kW')
read_and_print_value(modbus_reader, function=4, addr=0x11D1, size=2, value_type=TYPE_FLOAT, caption='Efficiency (COP)', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x11F1, size=1, value_type=TYPE_REGISTERS, caption='Selected year', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1201, size=1, value_type=TYPE_REGISTERS, caption='Selected type (Welectric:Wthermal:Wheating:Wdom.water:Wpool:beta_total:beta_heating:beta_d.water:beta_pool)', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1211, size=2, value_type=TYPE_FLOAT, caption='Selected value', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1231, size=2, value_type=TYPE_FLOAT, caption='Psourcepump', unit='kW')
read_and_print_value(modbus_reader, function=4, addr=0x1251, size=8, value_type=TYPE_STRING, caption='Firmware version', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x12D1, size=8, value_type=TYPE_STRING, caption='Firmware date', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1351, size=8, value_type=TYPE_STRING, caption='Model', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x13D1, size=8, value_type=TYPE_STRING, caption='Serial number', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1451, size=8, value_type=TYPE_STRING, caption='Set model', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x14D1, size=8, value_type=TYPE_STRING, caption='Set serial nr', unit='')
read_and_print_value(modbus_reader, function=1, addr=0x1551, size=8, value_type=TYPE_BITS, caption='Operating state', unit='')
read_and_print_value(modbus_reader, function=1, addr=0x1561, size=16, value_type=TYPE_BITS, caption='Digital output', unit='')
read_and_print_value(modbus_reader, function=1, addr=0x1571, size=8, value_type=TYPE_BITS, caption='Digital input', unit='')
read_and_print_value(modbus_reader, function=1, addr=0x1581, size=16, value_type=TYPE_BITS, caption='ADC error', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1591, size=8, value_type=TYPE_STRING, caption='LCD line 1', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1611, size=8, value_type=TYPE_STRING, caption='LCD line 2', unit='')
read_and_print_value(modbus_reader, function=4, addr=0x1691, size=2, value_type=TYPE_FLOAT, caption='Time factor', unit='')


# In[ ]:


# Wichtige Werte (für Home Assistant)

modbus_reader = ModbusReader('192.168.68.200')
    
read_and_print_value(modbus_reader, function=3, addr=0x01B1, size=2, value_type=TYPE_FLOAT32, caption='Outdoor Temp.', unit="°C")
read_and_print_value(modbus_reader, function=3, addr=0x01D1, size=2, value_type=TYPE_FLOAT32, caption='Outdoor Temp. (1h)', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x01F1, size=2, value_type=TYPE_FLOAT32, caption='Outdoor Temp. (24h)', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0211, size=2, value_type=TYPE_FLOAT32, caption='Source in Temp', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0231, size=2, value_type=TYPE_FLOAT32, caption='Source out Temp', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0251, size=2, value_type=TYPE_FLOAT32, caption='Verdampfer', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0271, size=2, value_type=TYPE_FLOAT32, caption='Saugleitung', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0291, size=2, value_type=TYPE_FLOAT32, caption='Verdampfer Druck', unit="bar")
read_and_print_value(modbus_reader, function=4, addr=0x02B1, size=2, value_type=TYPE_FLOAT32, caption='Rücklauf target', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x02D1, size=2, value_type=TYPE_FLOAT32, caption='Rücklauf real', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x02F1, size=2, value_type=TYPE_FLOAT32, caption='T heating out', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0311, size=2, value_type=TYPE_FLOAT32, caption='Kondensator', unit='°C')
read_and_print_value(modbus_reader, function=3, addr=0x0331, size=2, value_type=TYPE_FLOAT32, caption='Kondensator Druck', unit="bar")
read_and_print_value(modbus_reader, function=4, addr=0x0391, size=2, value_type=TYPE_FLOAT32, caption='Warmwasser', unit='°C')

print("Heizungswerte");
read_and_print_value(modbus_reader, function=1, addr=0x03F1, size=1, value_type=TYPE_BOOL, caption='Heating turn off')
read_and_print_value(modbus_reader, function=4, addr=0x0421, size=2, value_type=TYPE_FLOAT32, caption='T out begin')
read_and_print_value(modbus_reader, function=4, addr=0x0441, size=0x2, value_type=TYPE_FLOAT32, caption='T return base')
read_and_print_value(modbus_reader, function=4, addr=0x0461, size=0x2, value_type=TYPE_FLOAT32, caption='T curve gradient')
read_and_print_value(modbus_reader, function=4, addr=0x0481, size=0x2, value_type=TYPE_FLOAT32, caption='T curve limit')
read_and_print_value(modbus_reader, function=4, addr=0x04A1, size=0x2, value_type=TYPE_FLOAT32, caption='T return real')
read_and_print_value(modbus_reader, function=4, addr=0x04C1, size=0x2, value_type=TYPE_FLOAT32, caption='T return target')
read_and_print_value(modbus_reader, function=4, addr=0x04E1, size=0x2, value_type=TYPE_FLOAT32, caption='Heating Hysteresis')
read_and_print_value(modbus_reader, function=1, addr=0x0531, size=0x1, value_type=TYPE_BOOL, caption='Increase Off')
read_and_print_value(modbus_reader, function=4, addr=0x0541, size=0x1, value_type=TYPE_TIME, caption='Set pt chg start')
read_and_print_value(modbus_reader, function=4, addr=0x0551, size=0x1, value_type=TYPE_TIME, caption='Set pt chg stop')
read_and_print_value(modbus_reader, function=4, addr=0x0561, size=0x2, value_type=TYPE_FLOAT32, caption='Setpoint change')
read_and_print_value(modbus_reader, function=4, addr=0x0581, size=0x1, value_type=TYPE_UINT8, caption='Step mode')
read_and_print_value(modbus_reader, function=4, addr=0x0591, size=0x2, value_type=TYPE_FLOAT32, caption='Step max. diff.')

# Warmwasser
print("Warmwasser");
read_and_print_value(modbus_reader, function=1, addr=0x0661, size=1, value_type=TYPE_BOOL, caption='Hot Water turn off')
read_and_print_value(modbus_reader, function=4, addr=0x0691, size=2, value_type=TYPE_FLOAT32, caption='Real')
read_and_print_value(modbus_reader, function=4, addr=0x06B1, size=2, value_type=TYPE_FLOAT32, caption='Target')
read_and_print_value(modbus_reader, function=4, addr=0x06D1, size=2, value_type=TYPE_FLOAT32, caption='Domestic Water Hysteresis')


# OperatingHours
print("OperatingHours");
read_and_print_value(modbus_reader, function=4, addr=0x0A41, size=2, value_type=TYPE_FLOAT, caption='Compressor 1', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0A81, size=2, value_type=TYPE_FLOAT, caption='Heating comp.', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0AA1, size=2, value_type=TYPE_FLOAT, caption='Heating aux.', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0AE1, size=2, value_type=TYPE_FLOAT, caption='Hotwater comp.', unit='h')
read_and_print_value(modbus_reader, function=4, addr=0x0B01, size=2, value_type=TYPE_FLOAT, caption='Hotwater aux.', unit='h')


# Default Services
print("Default Services");
read_and_print_value(modbus_reader, function=4, addr=0x1191, size=2, value_type=TYPE_FLOAT, caption='P<sub>electric</sub>', unit='kW')
read_and_print_value(modbus_reader, function=4, addr=0x11B1, size=2, value_type=TYPE_FLOAT, caption='P<sub>thermal</sub>', unit='kW')
read_and_print_value(modbus_reader, function=4, addr=0x11D1, size=2, value_type=TYPE_FLOAT, caption='Efficiency (COP)', unit='')

read_and_print_value(modbus_reader, function=4, addr=0x1691, size=2, value_type=TYPE_FLOAT, caption='Time Factor', unit='')


# In[ ]:





# In[ ]:




